@@ -48,3 +48,22 @@ class ConfigClient(Client):
         """
         url = self.gi.url + '/version'
         return self._get(url=url)
+
+    def whoami(self):
+        """
+        Return information about the current authenticated user.
+
+        :rtype: dict
+        :return: Information about current authenticated user
+          For example::
+
+            {'active': True,
+             'deleted': False,
+             'email': 'user@example.org',
+             'id': '4aaaaa85aacc9caa',
+             'last_password_change': '2021-07-29T05:34:54.632345',
+             'model_class': 'User',
+             'username': 'julia'}
+        """
+        url = self.gi.url + "/whoami"
+        return self._get(url=url)
