@@ -26,7 +26,7 @@ class JobsClient(Client):
 
     def get_jobs(self, state=None, history_id=None, invocation_id=None, tool_id=None,
                  workflow_id=None, user_id=None, date_range_min=None, date_range_max=None,
-                 limit=500, offset=0, user_details=False):
+                 limit=500, offset=0, user_details=False, order_by=None):
         """
         Get all jobs, or select a subset by specifying optional arguments for
         filtering (e.g. a state).
@@ -73,6 +73,10 @@ class JobsClient(Client):
         :param user_details: If ``True`` and the user is an admin, add the user
           email to each returned job dictionary.
 
+        :type order_by: str
+        :param order_by: Whether to order jobs by ``create_time`` or
+          ``update_time`` (the default).
+
         :rtype: list of dict
         :return: Summary information for each selected job.
           For example::
@@ -93,8 +97,8 @@ class JobsClient(Client):
               'update_time': '2014-03-01T16:05:39.558458'}]
 
         .. note::
-          The following filtering options can only be used with Galaxy ``release_21.05`` or later:
-            user_id, limit, offset, workflow_id, invocation_id
+          The following options work only on Galaxy 21.05 or later: ``user_id``,
+          ``limit``, ``offset``, ``workflow_id``, ``invocation_id``.
         """
         params = {
             'limit': limit,
@@ -118,6 +122,8 @@ class JobsClient(Client):
             params['date_range_max'] = date_range_max
         if user_details:
             params['user_details'] = user_details
+        if order_by:
+            params['order_by'] = order_by
         return self._get(params=params)
 
     def show_job(self, job_id, full_details=False):
@@ -185,7 +191,10 @@ class JobsClient(Client):
 
         :type tool_inputs_update: dict
         :param tool_inputs_update: dictionary specifying any changes which should be
-          made to tool parameters for the rerun job.
+          made to tool parameters for the rerun job. This dictionary should have the same
+          structure as is required when submitting the ``tool_inputs`` dictionary to
+          ``gi.tools.run_tool()``, but only needs to include the inputs or parameters
+          to be updated for the rerun job.
 
         :type history_id: str
         :param history_id: ID of the history in which the job should be executed; if
@@ -195,7 +204,7 @@ class JobsClient(Client):
         :return: Information about outputs and the rerun job
 
         .. note::
-          This method can only be used with Galaxy ``release_21.01`` or later.
+          This method works only on Galaxy 21.01 or later.
         """
         job_rerun_params = self._build_for_rerun(job_id)
         job_inputs = job_rerun_params['state_inputs']
@@ -205,9 +214,18 @@ class JobsClient(Client):
                 raise ValueError('remap was set to True, but this job is not remappable.')
             job_inputs['rerun_remap_job_id'] = job_id
 
-        if tool_inputs_update:
+        def update_inputs(inputs, tool_inputs_update):
+            # recursively update inputs with tool_inputs_update
             for input_param, input_value in tool_inputs_update.items():
-                job_inputs[input_param] = input_value
+                if type(input_value) is dict:
+                    update_inputs(inputs[input_param], input_value)
+                else:
+                    inputs[input_param] = input_value
+            return inputs
+
+        if tool_inputs_update:
+            update_inputs(job_inputs, tool_inputs_update)
+
         url = '/'.join((self.gi.url, 'tools'))
         payload = {
             "history_id": history_id if history_id else job_rerun_params['history_id'],
@@ -259,7 +277,7 @@ class JobsClient(Client):
           ``inputs`` and ``state``.
 
         .. note::
-          This method is only supported by Galaxy 18.01 or later.
+          This method works only on Galaxy 18.01 or later.
         """
         job_info = {
             'tool_id': tool_id,
@@ -324,7 +342,7 @@ class JobsClient(Client):
         :return: dict containing job error reply
 
         .. note::
-          This method is only supported by Galaxy 20.01 or later.
+          This method works only on Galaxy 20.01 or later.
         """
         payload = {
             "message": message,
@@ -348,7 +366,7 @@ class JobsClient(Client):
         :return: dict containing potential problems
 
         .. note::
-          This method is only supported by Galaxy 19.05 or later.
+          This method works only on Galaxy 19.05 or later.
         """
         url = self._make_url(module_id=job_id) + '/common_problems'
         return self._get(url=url)
@@ -390,10 +408,10 @@ class JobsClient(Client):
         :return: dict containing output dataset associations
 
         .. note::
-          This method is only supported by Galaxy 18.09 or later.
+          This method works only on Galaxy 18.09 or later.
         """
         url = self._make_url(module_id=job_id) + '/resume'
-        return self._put(url=url, payload={})
+        return self._put(url=url)
 
     def get_destination_params(self, job_id: str) -> dict:
         """
@@ -407,8 +425,8 @@ class JobsClient(Client):
         :return: Destination parameters for the given job
 
         .. note::
-          This method is only supported by Galaxy 20.05 or later and requires
-          the user to be an admin.
+          This method works only on Galaxy 20.05 or later and if the user is a
+          Galaxy admin.
         """
         url = self._make_url(module_id=job_id) + '/destination_params'
         return self._get(url=url)
@@ -422,8 +440,8 @@ class JobsClient(Client):
         :return: Status of the job lock
 
         .. note::
-          This method is only supported by Galaxy 20.05 or later and requires
-          the user to be an admin.
+          This method works only on Galaxy 20.05 or later and if the user is a
+          Galaxy admin.
         """
         url = self.gi.url + '/job_lock'
         response = self._get(url=url)
@@ -439,8 +457,8 @@ class JobsClient(Client):
         :return: Updated status of the job lock
 
         .. note::
-          This method is only supported by Galaxy 20.05 or later and requires
-          the user to be an admin.
+          This method works only on Galaxy 20.05 or later and if the user is a
+          Galaxy admin.
         """
         payload = {
             'active': active,
