@@ -6,6 +6,7 @@ should not use it directly.
 A base representation of an instance
 """
 import base64
+import contextlib
 import json
 import logging
 from urllib.parse import (
@@ -38,7 +39,7 @@ class GalaxyClient:
             # Try to guess the scheme, starting from the more secure
             for scheme in ('https://', 'http://'):
                 log.warning(f"Missing scheme in url, trying with {scheme}")
-                try:
+                with contextlib.suppress(requests.RequestException):
                     r = requests.get(
                         scheme + url,
                         timeout=self.timeout,
@@ -47,8 +48,6 @@ class GalaxyClient:
                     r.raise_for_status()
                     found_scheme = scheme
                     break
-                except Exception:
-                    pass
             else:
                 raise ValueError(f"Missing scheme in url {url}")
             url = found_scheme + url
@@ -63,10 +62,9 @@ class GalaxyClient:
             self._key = None
             self.email = email
             self.password = password
-        self.json_headers = {
-            'Content-Type': 'application/json',
-            'x-api-key': self.key,
-        }
+        self.json_headers = {'Content-Type': 'application/json'}
+        # json_headers needs to be set before key can be defined, otherwise authentication with email/password causes an error
+        self.json_headers['x-api-key'] = self.key
 
     def make_get_request(self, url, **kwargs):
         """
@@ -90,7 +88,7 @@ class GalaxyClient:
         r = requests.get(url, headers=headers, **kwargs)
         return r
 
-    def make_post_request(self, url, payload, params=None, files_attached=False):
+    def make_post_request(self, url, payload=None, params=None, files_attached=False):
         """
         Make a POST request using the provided ``url`` and ``payload``.
         The ``payload`` must be a dict that contains the request values.
@@ -111,7 +109,7 @@ class GalaxyClient:
             not of type ``FileStream``.
             """
             for k, v in d.items():
-                if not isinstance(v, FileStream):
+                if not isinstance(v, (FileStream, str, bytes)):
                     d[k] = json.dumps(v)
             return d
 
@@ -119,15 +117,16 @@ class GalaxyClient:
         # leveraging the requests-toolbelt library if any files have
         # been attached.
         if files_attached:
-            payload = my_dumps(payload)
             if params:
                 payload.update(params)
+            payload = my_dumps(payload)
             payload = MultipartEncoder(fields=payload)
             headers = self.json_headers.copy()
             headers['Content-Type'] = payload.content_type
             post_params = None
         else:
-            payload = json.dumps(payload)
+            if payload is not None:
+                payload = json.dumps(payload)
             headers = self.json_headers
             post_params = params
 
@@ -195,7 +194,8 @@ class GalaxyClient:
 
         :return: The decoded response.
         """
-        payload = json.dumps(payload)
+        if payload is not None:
+            payload = json.dumps(payload)
         headers = self.json_headers
         r = requests.put(
             url,
@@ -231,7 +231,8 @@ class GalaxyClient:
 
         :return: The decoded response.
         """
-        payload = json.dumps(payload)
+        if payload is not None:
+            payload = json.dumps(payload)
         headers = self.json_headers
         r = requests.patch(
             url,
